@@ -7,12 +7,6 @@
 # for license details.
 """
 """
-try:
-    import CoolProp
-except:    
-    add_ID = True
-else:
-    add_ID = False
 import thermosteam as tmo
 from warnings import warn
 from flexsolve import IQ_interpolation, aitken_secant
@@ -44,7 +38,7 @@ from chemicals.elements import (
 )
 from chemicals.dipole import dipole_moment
 from .free_energy import (
-    Enthalpy, Entropy,
+    Enthalpy, Entropy, EntropyGas,
     EnthalpyRefSolid, EnthalpyRefLiquid, EnthalpyRefGas,
     EntropyRefSolid, EntropyRefLiquid, EntropyRefGas,
     ExcessEnthalpyRefSolid, ExcessEnthalpyRefLiquid, ExcessEnthalpyRefGas,
@@ -67,7 +61,7 @@ from . import functional as fn
 from ._phase import check_phase, valid_phases
 from . import units_of_measure as thermo_units
 from chemicals.utils import Z
-from thermo.eos import IG, PR
+from thermo.eos import IG, PR, SRK
 from thermo import (
     TDependentProperty, TPDependentProperty,
     VaporPressure, 
@@ -102,7 +96,7 @@ sugar_solid_densities = {
 }
 
 class CompressibilityFactor:
-    __slots__ = ('V')
+    __slots__ = ('V',)
     
     def __init__(self, V):
         self.V = V
@@ -373,10 +367,10 @@ class Chemical:
         
     >>> # Vapor pressure (Pa)
     >>> Water.Psat(T=373.15)
-    101284.55
+    101417.99665995422
     >>> # Surface tension (N/m)
     >>> Water.sigma(T=298.15)
-    0.07205
+    0.07197220523022962
     >>> # Molar volume (m^3/mol)
     >>> Water.V(phase='l', T=298.15, P=101325)
     1.806...e-05
@@ -401,16 +395,14 @@ class Chemical:
     Temperature dependent properties are managed by objects:
     
     >>> Water.Psat
-    VaporPressure(CASRN="7732-18-5 (Water)", Tb=373.124, Tc=647.14, 
-                  Pc=22048320.0, omega=0.344, extrapolation="AntoineAB|DIPPR101_ABC",
-                  method="WAGNER_MCGARRY")
+    VaporPressure(CASRN="7732-18-5", Tb=373.124, Tc=647.14, Pc=22048320.0, omega=0.344, extrapolation="AntoineAB|DIPPR101_ABC", method="IAPWS")
 
     Phase dependent properties have attributes with model handles for each phase:
 
     >>> Water.V
     <PhaseTPHandle(phase, T, P) -> V [m^3/mol]>
     >>> # Water.V.l
-    >>> # VolumeLiquid(CASRN="7732-18-5 (Water)", MW=18.01528, Tb=373.124, Tc=647.14, Pc=22048320.0, Vc=5.6-05, Zc=0.2294727, omega=0.344, dipole=1.85, Psat=VaporPressure(CASRN="7732-18-5 (Water)", Tb=373.124, Tc=647.14, Pc=22048320.0, omega=0.344, extrapolation="AntoineAB|DIPPR101_ABC", method="WAGNER_MCGARRY"), eos=[PR(Tc=647.14, Pc=22048320.0, omega=0.344, T=298.15, P=101325.0)], extrapolation="constant", method="VDI_PPDS", method_P="COSTALD_COMPRESSED", tabular_extrapolation_permitted=True)
+    >>> # VolumeLiquid(CASRN="7732-18-5", MW=18.01528, Tb=373.124, Tc=647.14, Pc=22048320.0, Vc=5.6-05, Zc=0.2294727, omega=0.344, dipole=1.85, Psat=VaporPressure(CASRN="7732-18-5", Tb=373.124, Tc=647.14, Pc=22048320.0, omega=0.344, extrapolation="AntoineAB|DIPPR101_ABC", method="WAGNER_MCGARRY"), eos=[PR(Tc=647.14, Pc=22048320.0, omega=0.344, T=298.15, P=101325.0)], extrapolation="constant", method="VDI_PPDS", method_P="COSTALD_COMPRESSED", tabular_extrapolation_permitted=True)
     
     A new model can be added easily using `add_method`, for example:
         
@@ -418,7 +410,7 @@ class Chemical:
     ...     return 10.0**(10.116 -  1687.537 / (T - 42.98))
     >>> Water.Psat.add_method(f=User_antoine_model, Tmin=273.20, Tmax=473.20)
     >>> Water.Psat
-    VaporPressure(CASRN="7732-18-5 (Water)", Tb=373.124, Tc=647.14, Pc=22048320.0, omega=0.344, extrapolation="AntoineAB|DIPPR101_ABC", method="USER_METHOD")
+    VaporPressure(CASRN="7732-18-5", Tb=373.124, Tc=647.14, Pc=22048320.0, omega=0.344, extrapolation="AntoineAB|DIPPR101_ABC", method="USER_METHOD")
 
     The `add_method` method is a high level interface that even lets you create a constant model:
         
@@ -482,6 +474,8 @@ class Chemical:
                  *_handles, *_data,
                  '_N_solutes')
     
+    #: [thermo.GEOS] Default EOS for pure components
+    EOS_default = PR
     #: [float] Reference temperature in Kelvin
     T_ref = 298.15
     #: [float] Reference pressure in Pascal
@@ -515,7 +509,6 @@ class Chemical:
                 raise ValueError(f'invalid phase {repr(phase)} encountered while parsing ID')
             ID = ID[:-2]
         search_ID = search_ID or ID
-        if not eos: eos = PR
         if search_db:
             metadata = pubchem_db.search(search_ID)
             data['metadata'] = metadata
@@ -554,7 +547,7 @@ class Chemical:
         return self
 
     @classmethod
-    def new(cls, ID, CAS, eos=PR, phase_ref=None, phase=None, **data):
+    def new(cls, ID, CAS, eos=None, phase_ref=None, phase=None, **data):
         """Create a new chemical from data without searching through
         the database, and load all possible models from given data."""
         self = super().__new__(cls)
@@ -643,7 +636,7 @@ class Chemical:
         self._CAS = CAS or ID
         for i,j in data.items(): setfield(self, '_' + i , j)
         if formula: self.formula = formula
-        self._eos = create_eos(PR, self._Tc, self._Pc, self._omega)
+        self._eos = create_eos(self.EOS_default, self._Tc, self._Pc, self._omega)
         TDependentProperty.RAISE_PROPERTY_CALCULATION_ERROR = False
         self._estimate_missing_properties(None)
         self._init_handles(CAS, self._MW, self._Tm, self._Tb, self._Pt, self._Tt, 
@@ -689,6 +682,7 @@ class Chemical:
             setfield(new, field, None)
         new._ID = ID
         new._CAS = CAS or ID
+        new._aliases = set()
         TDependentProperty.RAISE_PROPERTY_CALCULATION_ERROR = False
         new._init_energies(new.Cn, new.Hvap, new.Psat, new.Hfus, new.Sfus, new.Tm,
                            new.Tb, new.eos, new.phase_ref)
@@ -702,14 +696,10 @@ class Chemical:
         handles = (self._Psat, self._Hvap, self._sigma, self._epsilon,
                    self._V, self._Cn, self._mu, self._kappa)
         isa = isinstance
-        if add_ID and self.CAS != self.ID:
-            label = f"{self.CAS} ({self.ID})"
-        else:
-            label = self.CAS
+        label = self.CAS
         for handle in handles:
             if isa(handle, PhaseHandle):
-                for i, j in handle:
-                    j.CASRN = label
+                for i, j in handle: j.CASRN = label
             else:
                 handle.CASRN = label
 
@@ -810,7 +800,7 @@ class Chemical:
         >>> import thermosteam as tmo
         >>> Water = tmo.Chemical('Water', cache=True)
         >>> Water.get_property('sigma', 'N/m', 300.) # Surface tension
-        0.07176
+        0.07168596252716256
 
         >>> Water.get_property('rho', 'g/cm3', 'l', 300., 101325) # Density
         0.9962
@@ -1297,10 +1287,14 @@ class Chemical:
     
     ### Data solvers ###
 
-    def Tsat(self, P, Tguess=None, Tmin=None, Tmax=None):
+    def Tsat(self, P, Tguess=None, Tmin=None, Tmax=None, *, check_validity=True):
         """Return the saturated temperature (in Kelvin) given the pressure (in Pascal)."""
         Tb = self._Tb
         Psat = self._Psat
+        if check_validity and P >= self._Pc:
+            raise ValueError(
+                'pressure, P, cannot be above the supercritical pressure, Pc'
+            )
         if not Psat: return None
         if not Tmin: Tmin = Psat.Tmin + 1.
         if not Tmax: Tmax = Psat.Tmax - 1.
@@ -1322,7 +1316,7 @@ class Chemical:
 
     ### Reinitializers ###
     
-    def reset(self, CAS, eos=PR, phase_ref=None,
+    def reset(self, CAS, eos=None, phase_ref=None,
               smiles=None, InChI=None, InChI_key=None,
               pubchemid=None, iupac_name=None, common_name=None,
               formula=None, MW=None, Tm=None,
@@ -1407,7 +1401,7 @@ class Chemical:
     def reset_free_energies(self):
         """Reset the `H`, `S`, `H_excess`, and `S_excess` functors."""
         if not self._eos:
-            self._eos = create_eos(PR, self._Tc, self._Pc, self._omega)
+            self._eos = create_eos(self.EOS_default, self._Tc, self._Pc, self._omega)
         TDependentProperty.RAISE_PROPERTY_CALCULATION_ERROR = False
         self._init_energies(self._Cn, self._Hvap, self._Psat, self._Hfus, self._Sfus,
                             self._Tm, self._Tb, self._eos, self._phase_ref)
@@ -1501,7 +1495,7 @@ class Chemical:
         self._combustion = combustion
 
     def _init_eos(self, eos, Tc, Pc, omega):
-        self._eos = create_eos(eos, Tc, Pc, omega)
+        self._eos = create_eos(eos or self.EOS_default, Tc, Pc, omega)
 
     def _init_handles(self, CAS, MW, Tm, Tb, Pt, Tt, Tc, Pc, Zc, Vc, omega,
                       dipole, similarity_variable, iscyclic_aliphatic, eos,
@@ -1595,12 +1589,9 @@ class Chemical:
             elif not Pc: self._Pc = third_property
             elif not Vc: self._Vc = third_property
         Tb = self._Tb
-        if not Tb:
-            try:
-                self._Tb = Tb = self.Tsat(101325)
-            except:
-                self._Tb = Tb = None
-            
+        if not Tb and Pc is not None and 101325. < Pc:
+            try: self._Tb = Tb = self.Tsat(101325)
+            except: pass
         omega = self._omega
         if not omega and Pc and Tc:
             if Psat: 
@@ -1628,9 +1619,18 @@ class Chemical:
         elif Cn and single_phase:
             self._phase_ref = single_phase
             self._H = Enthalpy.functor(Cn, T_ref, H_ref)
-            self._S = Entropy.functor(Cn, T_ref, S0)
+            if single_phase == 'g':
+                self._S = EntropyGas.functor(Cn, T_ref, P_ref, S0)
+            else:
+                self._S = Entropy.functor(Cn, T_ref, P_ref, S0)
             Cn_s = Cn_l = Cn_g = Cn
-            has_Cns = has_Cnl = has_Cng = True
+            has_Cns = has_Cnl = has_Cng = False
+            if single_phase == 'l':
+                has_Cnl = True
+            elif single_phase == 's':
+                has_Cns = True
+            elif single_phase == 'g':
+                has_Cng = True    
         else:
             has_Cns = has_Cnl = has_Cng = False
         if phase_ref: phase_ref = phase_ref[0]
@@ -1673,8 +1673,11 @@ class Chemical:
                     else:
                         H_int_T_ref_to_Tb_l = S_int_T_ref_to_Tb_l = None
                     if Tm:
-                        H_int_Tm_to_T_ref_l = Cn_l.T_dependent_property_integral(Tm, T_ref)
-                        S_int_Tm_to_T_ref_l = Cn_l.T_dependent_property_integral_over_T(Tm, T_ref)
+                        try:
+                            H_int_Tm_to_T_ref_l = Cn_l.T_dependent_property_integral(Tm, T_ref)
+                            S_int_Tm_to_T_ref_l = Cn_l.T_dependent_property_integral_over_T(Tm, T_ref)
+                        except:
+                            H_int_Tm_to_T_ref_l = S_int_Tm_to_T_ref_l = None
                     else:
                         H_int_Tm_to_T_ref_l = S_int_Tm_to_T_ref_l = None
                 else:
@@ -1892,7 +1895,7 @@ class Chemical:
         if 'phase_ref' in properties:
             self._phase_ref = 'l'
         if 'eos' in properties:
-            self._eos = create_eos(PR, self._Tc, self._Pc, self._omega)
+            self._eos = create_eos(self.EOS_default, self._Tc, self._Pc, self._omega)
         if 'Cn' in properties:
             MW = self._MW
             Cn = self._Cn
@@ -2130,4 +2133,4 @@ def lock_phase(chemical, phase):
 # # Analytical
 # self.RI, self.RIT = refractive_index(CASRN=CAS)
 # self.conductivity, self.conductivityT = conductivity(CASRN=CAS)
-    
\ No newline at end of file
+    
