@@ -8,9 +8,10 @@
 """
 """
 from . import utils
-from .exceptions import UndefinedChemical
+from .exceptions import UndefinedChemicalAlias
 from ._chemical import Chemical
 from .indexer import ChemicalIndexer, SplitIndexer
+from collections.abc import Sequence
 import thermosteam as tmo
 import numpy as np
 
@@ -107,11 +108,11 @@ class Chemicals:
     >>> 'Butane' in chemicals
     False
     
-    An attempt to access a non-existent chemical raises an UndefinedChemical error:
+    An attempt to access a non-existent chemical raises an UndefinedChemicalAlias error:
     
     >>> chemicals['Butane']
     Traceback (most recent call last):
-    UndefinedChemical: 'Butane'
+    UndefinedChemicalAlias: 'Butane'
     
     """
     def __new__(cls, chemicals, cache=False):
@@ -153,7 +154,7 @@ class Chemicals:
             else:
                 return [dct[i] for i in key]
         except KeyError as key_error:
-            raise UndefinedChemical(key_error.args[0])
+            raise UndefinedChemicalAlias(key_error.args[0])
     
     def copy(self):
         """Return a copy."""
@@ -163,7 +164,9 @@ class Chemicals:
     
     def append(self, chemical):
         """Append a Chemical."""
-        if not isinstance(chemical, Chemical):
+        if isinstance(chemical, str):
+            chemical = Chemical(chemical)
+        elif not isinstance(chemical, Chemical):
             raise TypeError("only 'Chemical' objects can be appended, "
                            f"not '{type(chemical).__name__}'")
         ID = chemical.ID
@@ -343,6 +346,9 @@ class CompiledChemicals(Chemicals):
             self = cache[chemicals_tuple] = chemicals
         return self
     
+    def __hash__(self):
+        return hash(self.IDs)
+    
     def __dir__(self):
         return ('append', 'array', 'compile', 'extend', 
                 'get_combustion_reactions', 'get_index',
@@ -404,7 +410,7 @@ class CompiledChemicals(Chemicals):
         >>> s2.imol['l', 'Alcohol']
         0.678
         >>> s2.imol['l', ('Water', 'Alcohol')]
-        array([1.322, 0.678])
+        array([1.321, 0.679])
         
         Because groups are defined with a composition we can set values by 
         groups as well:
@@ -579,7 +585,7 @@ class CompiledChemicals(Chemicals):
                     raise Exception('chemical locked state has an invalid phase')
             else:
                 vle_chemicals.append(i)
-                lle_chemicals.append(i)
+                if i.Dortmund or i.UNIFAC or i.NIST or i.PSRK: lle_chemicals.append(i)
         dct['vle_chemicals'] = tuple_(vle_chemicals)
         dct['lle_chemicals'] = tuple_(lle_chemicals)
         dct['heavy_chemicals'] = tuple_(heavy_chemicals)
@@ -943,11 +949,11 @@ class CompiledChemicals(Chemicals):
         
         >>> chemicals.index('7732-18-5')
         0
-
+            
         """
         try: return self._index[ID]
         except KeyError:
-            raise UndefinedChemical(ID)
+            raise UndefinedChemicalAlias(ID)
 
     def indices(self, IDs):
         """
@@ -973,11 +979,11 @@ class CompiledChemicals(Chemicals):
         [0, 1]
 
         """
+        dct = self._index
         try:
-            dct = self._index
             return [dct[i] for i in IDs]
         except KeyError as key_error:
-            raise UndefinedChemical(key_error.args[0])
+            raise UndefinedChemicalAlias(key_error.args[0])
     
     def available_indices(self, IDs):
         """
@@ -1021,7 +1027,7 @@ class CompiledChemicals(Chemicals):
 
         Examples
         --------
-        Get multiple indices with a tuple of IDs:
+        Get multiple indices with a tuple/list of IDs:
         
         >>> from thermosteam import CompiledChemicals
         >>> chemicals = CompiledChemicals(['Water', 'Ethanol'], cache=True)
@@ -1039,27 +1045,28 @@ class CompiledChemicals(Chemicals):
         >>> chemicals.get_index(...)
         slice(None, None, None)
 
-        Anything else returns an error:
+        Collections (without an order) raise an error:
         
-        >>> chemicals.get_index(['Water', 'Ethanol'])
+        >>> chemicals.get_index({'Water', 'Ethanol'})
         Traceback (most recent call last):
-        TypeError: only strings, tuples, and ellipsis are valid index keys
+        TypeError: only strings, sequences, and ellipsis are valid index keys
 
         """
-        if isinstance(IDs, str):
+        if isinstance(IDs, (str, Chemical)):
             return self.index(IDs)
-        elif isinstance(IDs, tuple):
-            return self.indices(IDs)
         elif IDs is ...:
             return slice(None)
+        elif isinstance(IDs, Sequence):
+            return self.indices(IDs)
         else: # pragma: no cover
-            raise TypeError("only strings, tuples, and ellipsis are valid index keys")    
+            raise TypeError("only strings, chemicals, sequences, and ellipsis are valid index keys")    
     
     def _get_index_and_kind(self, key):
         index_cache = self._index_cache
-        if key in index_cache:
+        try:
+            if key.__hash__ is None: key = tuple(key)
             return index_cache[key]
-        else:
+        except KeyError:
             isa = isinstance
             kind = 0 # [int] Kind of index: 0 - normal, 1 - chemical group, 2 - nested chemical group
             if isa(key, str):
@@ -1072,8 +1079,10 @@ class CompiledChemicals(Chemicals):
             elif key is ...:
                 index = slice(None)
             else: # pragma: no cover
-                raise TypeError("only strings, tuples, and ellipsis are valid index keys")
+                raise TypeError("only strings, sequences of strings, and ellipsis are valid index keys")
             index_cache[key] = index, kind
+        except TypeError:
+            raise TypeError("only strings, sequences of strings, and ellipsis are valid index keys")
         return index, kind
     
     def __len__(self):
@@ -1123,4 +1132,4 @@ class CompiledChemicals(Chemicals):
         return [i for i, j in enumerate(self._has_lle & nonzeros) if j]
     
     def __repr__(self):
-        return f"{type(self).__name__}([{', '.join(self.IDs)}])"
\ No newline at end of file
+        return f"{type(self).__name__}([{', '.join(self.IDs)}])"
