@@ -7,6 +7,8 @@
 # for license details.
 """
 """
+from __future__ import annotations
+from ._thermo import Thermo
 from ._stream import Stream
 from ._thermal_condition import ThermalCondition
 from .indexer import MolarFlowIndexer
@@ -14,6 +16,10 @@ from ._phase import phase_tuple
 from . import equilibrium as eq
 from . import utils
 import numpy as np
+from typing import TYPE_CHECKING
+if TYPE_CHECKING:
+    from numpy.typing import NDArray
+    from typing import Optional, Sequence, Dict, Tuple
 
 __all__ = ('MultiStream', )
 
@@ -37,26 +43,29 @@ class MultiStream(Stream):
 
     Parameters
     ----------
-    ID : str, optional
+    ID : 
         A unique identification. If ID is None, stream will not be registered.
         If no ID is given, stream will be registered with a unique ID.
-    flow : 2d array, optional
+    flow : 
         All flow rates corresponding to `phases` by row and chemical IDs by column.
-    T : float
+    T : 
         Temperature [K]. Defaults to 298.15.
-    P : float
+    P : 
         Pressure [Pa]. Defaults to 101325.
-    phases : tuple['g', 'l', 's', 'G', 'L', 'S']
+    phases : 
         Tuple denoting the phases present. Defaults to ('g', 'l').
-    units : str, optional
+    units : 
         Flow rate units of measure (only mass, molar, and
         volumetric flow rates are valid). Defaults to 'kmol/hr'.
-    price : float, optional
+    price : 
         Price per unit mass [USD/kg]. Defaults to 0.
-    total_flow : float, optional
+    total_flow : 
         Total flow rate.
-    thermo : :class:`~thermosteam.Thermo`
+    thermo : 
         Thermodynamic equilibrium package. Defaults to `thermosteam.settings.get_thermo()`.
+    vlle :
+        Whether to run rigorous phase equilibrium to determine phases. 
+        Defaults to False.
     **phase_flow : tuple[str, float]
         phase-(ID, flow) pairs.
     
@@ -169,13 +178,13 @@ class MultiStream(Stream):
     >>> s1['l'].show()
     Stream: 
      phase: 'l', T: 365 K, P: 101325 Pa
-     flow (kmol/hr): Water    0.619
-                     Ethanol  0.0238
+     flow (kmol/hr): Water    0.616
+                     Ethanol  0.0235
     >>> s1['g'].show()
     Stream: 
      phase: 'g', T: 365 K, P: 101325 Pa
-     flow (kmol/hr): Water    0.491
-                     Ethanol  0.193
+     flow (kmol/hr): Water    0.494
+                     Ethanol  0.194
     
     Note that the phase cannot be changed:
     
@@ -185,9 +194,19 @@ class MultiStream(Stream):
     
     """
     __slots__ = ()
-    def __init__(self, ID="", flow=(), T=298.15, P=101325., phases=None, 
-                 units=None, price=0, total_flow=None, thermo=None, 
-                 characterization_factors=None, **phase_flows):
+    def __init__(self, 
+                 ID: Optional[str]="",
+                 flow: Optional[Sequence]=(), 
+                 T: Optional[float]=298.15, 
+                 P: Optional[float]=101325.,
+                 phases: Optional[Sequence[str]]=None, 
+                 units: Optional[str]=None, 
+                 price: Optional[float]=0,
+                 total_flow: Optional[float]=None, 
+                 thermo: Optional[Thermo]=None, 
+                 characterization_factors: Optional[Dict[str, float]]=None, 
+                 vlle: Optional[bool]=False, 
+                 **phase_flows: Tuple[str, float]):
         self.characterization_factors = {} if characterization_factors is None else {}
         self._thermal_condition = ThermalCondition(T, P)
         thermo = self._load_thermo(thermo)
@@ -229,8 +248,8 @@ class MultiStream(Stream):
         self._sink = self._source = None
         self.reset_cache()
         self._register(ID)
-        self._islinked = False
         self._user_equilibrium = None
+        if vlle: self.vlle(T, P)
         
     def reset_flow(self, total_flow=None, units=None, phases=None, **phase_flows):
         """
@@ -274,27 +293,36 @@ class MultiStream(Stream):
                 imol = MolarFlowIndexer.from_data(flow, phases, chemicals)
         self._imol = imol
     
-    def _get_property_cache(self, name, flow=False):
+    def _get_property(self, name, flow=False, nophase=False):
         property_cache = self._property_cache
         thermal_condition = self._thermal_condition
         imol = self._imol
         data = imol._data
         total = data.sum()
-        if total == 0.: return 0.
-        composition = data / total
-        literal = (imol._phases, thermal_condition._T, thermal_condition._P)
-        last_literal, last_composition, last_total = self._property_cache_key
-        if literal == last_literal and (composition - last_composition).all():
-            prop = property_cache.get(name)
-            if not prop: return prop
-            if flow:
-                return prop * total / last_total
-            else:
-                return prop
+        if total == 0.: 
+            return 0. if flow else None
         else:
-            self._property_cache_key = (literal, composition, total)
-            property_cache.clear()
-            return None
+            composition = data / total
+            if nophase:
+                literal = (thermal_condition._T, thermal_condition._P)
+            else:
+                literal = (imol._phases, thermal_condition._T, thermal_condition._P)
+            last_literal, last_composition = self._property_cache_key
+            if name in property_cache and literal == last_literal and (composition == last_composition).all():
+                value = property_cache.get(name)
+            else:
+                self._property_cache_key = (literal, composition)
+                if nophase:
+                    calculate = getattr(self.mixture, name)
+                    self._property_cache[name] = value = calculate(
+                        composition.sum(axis=0), *self.thermal_condition
+                    )
+                else:
+                    calculate = getattr(self.mixture, 'x' + name)
+                    self._property_cache[name] = value = calculate(
+                        zip(imol._phases, composition), *self.thermal_condition
+                    )
+        return value * total if flow else value
     
     def reset_cache(self):
         """Reset cache regarding equilibrium methods."""
@@ -322,7 +350,6 @@ class MultiStream(Stream):
         else:
             stream = Stream.__new__(Stream)
             stream._ID = stream._sink = stream._source = None
-            stream._islinked = False
             stream._imol = self._imol.get_phase(phase)
             stream._thermal_condition = self._thermal_condition
             stream._thermo = self._thermo
@@ -330,7 +357,7 @@ class MultiStream(Stream):
             stream._dew_point_cache = self._dew_point_cache
             stream._property_cache = {}
             stream.characterization_factors = {}
-            stream._property_cache_key = None, None, None
+            stream._property_cache_key = None, None
             stream._user_equilibrium = None
             streams[phase] = stream
         return stream
@@ -381,17 +408,19 @@ class MultiStream(Stream):
     
     ### Property getters ###
     
-    def get_flow(self, units, key=...):
+    def get_flow(self, units: str, key=...):
         """
         Return an array of flow rates in requested units.
         
         Parameters
         ----------
-        units : str
+        units : 
             Units of measure.
-        key : tuple(phase, IDs), phase, or IDs
-            * phase: str, ellipsis, or missing.
-            * IDs: str, tuple(str), ellipisis, or missing.
+        key : 
+            Index key as (phase, IDs), phase, or IDs where:
+            
+            * phase is a str, ellipsis, or missing.
+            * IDs is a str, Sequence[str], ellipisis, or missing.
 
         Examples
         --------
@@ -406,19 +435,21 @@ class MultiStream(Stream):
         indexer = getattr(self, 'i' + name)
         return factor * indexer[key]
     
-    def set_flow(self, data, units, key=...):
+    def set_flow(self, data: NDArray[float]|float, units: str, key=...):
         """
         Set flow rates in given units.
 
         Parameters
         ----------
-        data : 1d ndarray or float
+        data : 
             Flow rate data.
-        units : str
+        units :
             Units of measure.
-        key : tuple(phase, IDs), phase, or IDs
-            * phase: str, ellipsis, or missing.
-            * IDs: str, tuple(str), ellipisis, or missing.
+        key : 
+            Index key as (phase, IDs), phase, or IDs where:
+            
+            * phase is a str, ellipsis, or missing.
+            * IDs is a str, Sequence[str], ellipisis, or missing.
 
         Examples
         --------
@@ -437,8 +468,8 @@ class MultiStream(Stream):
     ### Stream data ###
     
     @property
-    def phases(self):
-        """tuple[str] All phases avaiable."""
+    def phases(self) -> tuple[str, ...]:
+        """All phases avaiable."""
         return self._imol._phases
     @phases.setter
     def phases(self, phases):
@@ -452,20 +483,20 @@ class MultiStream(Stream):
     ### Flow properties ###
             
     @property
-    def mol(self):
-        """[Array] Chemical molar flow rates (total of all phases)."""
+    def mol(self) -> NDArray[float]:
+        """Chemical molar flow rates (total of all phases)."""
         mol = self._imol._data.sum(0)
         mol.setflags(0)
         return mol
     @property
-    def mass(self):
-        """[Array] Chemical mass flow rates (total of all phases)."""
+    def mass(self) -> NDArray[float]:
+        """Chemical mass flow rates (total of all phases)."""
         mass = self.mol * self.chemicals.MW
         mass.setflags(0)
         return mass
     @property
-    def vol(self):
-        """[Array] Chemical volumetric flow rates (total of all phases)."""
+    def vol(self) -> NDArray[float]:
+        """Chemical volumetric flow rates (total of all phases)."""
         vol = self.ivol._data.sum(0)
         vol.setflags(0)
         return vol
@@ -473,129 +504,118 @@ class MultiStream(Stream):
     ### Net flow properties ###
     
     @property
-    def H(self):
-        """[float] Enthalpy flow rate in kJ/hr."""
-        H = self._get_property_cache('H', True)
-        if H is None:
-            self._property_cache['H'] = H = self.mixture.xH(self._imol, *self._thermal_condition)
-        return H
+    def H(self) -> float:
+        """Enthalpy flow rate [kJ/hr]."""
+        return self._get_property('H', True)
     @H.setter
     def H(self, H):
-        self.T = self.mixture.xsolve_T(self._imol, H, *self._thermal_condition)
+        if not H and self.isempty(): return
+        self.T = self.mixture.xsolve_T_at_HP(
+            self._imol, H, *self._thermal_condition
+        )
 
     @property
-    def S(self):
-        """[float] Absolute entropy flow rate in kJ/hr."""
-        S = self._get_property_cache('S', True)
-        if S is None:
-            self._property_cache['S'] = S = self.mixture.xS(self._imol, *self._thermal_condition)
-        return S
+    def h(self) -> float:
+        """Specific enthalpy in kJ/kmol."""
+        return self._get_property('H')
+    @h.setter
+    def h(self, h: float):
+        if not h and self.isempty(): return
+        self.T = self.mixture.xsolve_T_at_HP(
+            self._imol.iter_composition(), h, *self._thermal_condition
+        )
+
     @property
-    def C(self):
-        """[float] Heat capacity flow rate in kJ/hr."""
-        C = self._get_property_cache('C', True)
-        if C is None:
-            self._property_cache['C'] = C = self.mixture.xCn(self._imol, self.T)
-        return C
-    @property
-    def F_vol(self):
-        """[float] Total volumetric flow rate in m3/hr."""
-        return 1000. * self.V * self.F_mol
-    @F_vol.setter
-    def F_vol(self, value):
-        F_vol = self.F_vol
-        if not F_vol: raise AttributeError("undefined composition; cannot set flow rate")
-        self.ivol._data[:] *= value / F_vol / 1000.
-    
-    @property
-    def Hvap(self):
-        """[float] Enthalpy of vaporization flow rate in kJ/hr."""
-        mol = self._imol['l']
-        T = self._thermal_condition._T
-        Hvap = self._get_property_cache('Hvap')
-        if Hvap is None:
-            self._property_cache['Hvap'] = Hvap = sum([
-                i*j.Hvap(T) for i,j in zip(mol, self.chemicals)
-                if i and not j.locked_state
-            ])
-        return Hvap
+    def S(self) -> float:
+        """Absolute entropy flow rate [kJ/hr]."""
+        return self._get_property('S', True)
+    @S.setter
+    def S(self, S):
+        if not S and self.isempty(): return
+        self.T = self.mixture.xsolve_T_at_SP(
+            self._imol, S, *self._thermal_condition
+        )
     
     ### Composition properties ###
     
     @property
-    def vapor_fraction(self):
+    def vapor_fraction(self) -> float:
         """Molar vapor fraction."""
         return get_phase_fraction(self, 'gG')
     @property
-    def liquid_fraction(self):
+    def liquid_fraction(self) -> float:
         """Molar liquid fraction."""
         return get_phase_fraction(self, 'lL')
     @property
-    def solid_fraction(self):
+    def solid_fraction(self) -> float:
         """Molar solid fraction."""
         return get_phase_fraction(self, 'sS')
-    
-    @property
-    def V(self):
-        """[float] Molar volume [m^3/mol]."""
-        V = self._get_property_cache('V')
-        if V is None:
-            self._property_cache['V'] = V = self.mixture.xV(self._imol.iter_composition(), *self._thermal_condition)
-        return V
-    @property
-    def kappa(self):
-        """[float] Thermal conductivity [W/m/k]."""
-        kappa = self._get_property_cache('kappa')
-        if kappa is None:
-            self._property_cache['kappa'] = kappa = self.mixture.xkappa(self._imol.iter_composition(), *self._thermal_condition)
-        return kappa      
-    @property
-    def Cn(self):
-        """[float] Molar heat capacity [J/mol/K]."""
-        Cn = self._get_property_cache('Cn')
-        if Cn is None:
-            self._property_cache['Cn'] = Cn = self.mixture.xCn(self._imol.iter_composition(), self.T)
-        return Cn
-    @property
-    def mu(self):
-        """[float] Hydrolic viscosity [Pa*s]."""
-        mu = self._get_property_cache('mu')
-        if mu is None:
-            self._property_cache['mu'] = mu = self.mixture.xmu(self._imol.iter_composition(), *self._thermal_condition)
-        return mu
-    @property
-    def sigma(self):
-        """[float] Surface tension [N/m]."""
-        mol = self._imol['l']
-        sigma = self._get_property_cache('sigma')
-        if sigma is None:
-            self._property_cache['sigma'] = sigma = self.mixture.sigma(mol / mol.sum(), *self._thermal_condition)
-        return sigma
-    @property
-    def epsilon(self):
-        """[float] Relative permittivity [-]."""
-        mol = self._imol['l']
-        epsilon = self._get_property_cache('epsilon')
-        if epsilon is None:
-            self._property_cache['epsilon'] = epsilon = self.mixture.epsilon(mol / mol.sum(), *self._thermal_condition)
-        return epsilon
         
     ### Methods ###
     
-    def copy_flow(self, other, phase=..., IDs=..., *, remove=False, exclude=False):
+    def split_to(self, s1, s2, split, energy_balance=True):
+        """
+        Split molar flow rate from this stream to two others given
+        the split fraction or an array of split fractions.
+        
+        Examples
+        --------
+        >>> import thermosteam as tmo
+        >>> chemicals = tmo.Chemicals(['Water', 'Ethanol'], cache=True)
+        >>> tmo.settings.set_thermo(chemicals)
+        >>> s = tmo.Stream('s', Water=20, Ethanol=10, units='kg/hr')
+        >>> s.phases = ('l', 'g')
+        >>> s1 = tmo.Stream('s1')
+        >>> s2 = tmo.Stream('s2')
+        >>> split = chemicals.kwarray(dict(Water=0.5, Ethanol=0.1))
+        >>> s.split_to(s1, s2, split)
+        >>> s1.show(flow='kg/hr')
+        MultiStream: s1
+         phases: ('g', 'l'), T: 298.15 K, P: 101325 Pa
+         flow (kg/hr): (l) Water    10
+                           Ethanol  1
+        
+        >>> s2.show(flow='kg/hr')
+        MultiStream: s2
+         phases: ('g', 'l'), T: 298.15 K, P: 101325 Pa
+         flow (kg/hr): (l) Water    10
+                           Ethanol  9
+        
+        
+        """
+        phases = self.phases
+        if energy_balance or isinstance(s1, MultiStream) or isinstance(s2, MultiStream):
+            s1.phases = phases
+            s2.phases = phases
+            for phase in phases: self[phase].split_to(s1[phase], s2[phase], split)
+        else:
+            Stream.split_to(self, s1, s2, split)
+            return
+        if energy_balance:
+            tc1 = s1._thermal_condition
+            tc2 = s2._thermal_condition
+            tc = self._thermal_condition
+            tc1._T = tc2._T = tc._T
+            tc1._P = tc2._P = tc._P
+    
+    def copy_flow(self, 
+                  other: Stream, 
+                  phase: Optional[Sequence[str]|str]=...,
+                  IDs: Optional[Sequence[str]|str]=..., *,
+                  remove: Optional[bool]=False,
+                  exclude: Optional[bool]=False):
         """
         Copy flow rates of another stream to self.
         
         Parameters
         ----------
-        other : Stream
+        other : 
             Flow rates will be copied from here.
-        phase : str or Ellipsis
-        IDs=None : iterable[str], defaults to all chemicals.
-            Chemical IDs. 
-        remove=False: bool, optional
+        IDs : 
+            Chemical IDs. Defaults to all chemicals.
+        remove : 
             If True, copied chemicals will be removed from `stream`.
-        exclude=False: bool, optional
+        exclude :
             If True, exclude designated chemicals when copying.
         
         Notes
@@ -738,13 +758,13 @@ class MultiStream(Stream):
                 data[other_phase_index, IDs_index] = other_data[IDs_index]
                 if remove: other_data[IDs_index] = 0.
     
-    def get_normalized_mol(self, IDs):
+    def get_normalized_mol(self, IDs: Sequence[str]):
         """
         Return normalized molar fractions of given chemicals. The sum of the result is always 1.
 
         Parameters
         ----------
-        IDs : tuple[str]
+        IDs : 
             IDs of chemicals to be normalized.
 
         Examples
@@ -760,13 +780,13 @@ class MultiStream(Stream):
         z /= z.sum()
         return z
     
-    def get_normalized_vol(self, IDs):
+    def get_normalized_vol(self, IDs: Sequence[str]):
         """
         Return normalized mass fractions of given chemicals. The sum of the result is always 1.
 
         Parameters
         ----------
-        IDs : tuple[str]
+        IDs : 
             IDs of chemicals to be normalized.
 
         Examples
@@ -782,13 +802,13 @@ class MultiStream(Stream):
         z /= z.sum()
         return z
     
-    def get_normalized_mass(self, IDs):
+    def get_normalized_mass(self, IDs: Sequence[str]):
         """
         Return normalized mass fractions of given chemicals. The sum of the result is always 1.
 
         Parameters
         ----------
-        IDs : tuple[str]
+        IDs : 
             IDs of chemicals to be normalized.
 
         Examples
@@ -804,13 +824,13 @@ class MultiStream(Stream):
         z /= z.sum()
         return z
     
-    def get_molar_composition(self, IDs):
+    def get_molar_composition(self, IDs: Sequence[str]):
         """
         Return molar composition of given chemicals.
 
         Parameters
         ----------
-        IDs : tuple[str]
+        IDs : 
             IDs of chemicals.
 
         Examples
@@ -824,13 +844,13 @@ class MultiStream(Stream):
         """
         return self.imol[..., IDs].sum(0)/self.F_mol
     
-    def get_mass_composition(self, IDs):
+    def get_mass_composition(self, IDs: Sequence[str]):
         """
         Return mass composition of given chemicals.
 
         Parameters
         ----------
-        IDs : tuple[str]
+        IDs : 
             IDs of chemicals.
 
         Examples
@@ -844,13 +864,13 @@ class MultiStream(Stream):
         """
         return self.imass[..., IDs].sum(0)/self.F_mass
     
-    def get_volumetric_composition(self, IDs):
+    def get_volumetric_composition(self, IDs: Sequence[str]):
         """
         Return volumetric composition of given chemicals.
 
         Parameters
         ----------
-        IDs : tuple[str]
+        IDs : 
             IDs of chemicals.
 
         Examples
@@ -864,13 +884,14 @@ class MultiStream(Stream):
         """
         return self.ivol[..., IDs].sum(0)/self.F_vol
     
-    def get_concentration(self, phase, IDs):
+    def get_concentration(self, phase: str, IDs: Sequence[str]|str, units: Optional[str]=None):
         """
         Return concentration of given chemicals in kmol/m3.
 
         Parameters
         ----------
-        IDs : tuple[str]
+        phase :
+        IDs : 
             IDs of chemicals.
 
         Examples
@@ -881,28 +902,53 @@ class MultiStream(Stream):
         >>> s1.get_concentration('l', ('Water', 'Ethanol'))
         array([27.672,  4.265])
 
+        >>> s1.get_concentration('l', ('Water', 'Ethanol'), 'g/L')
+        array([498.512, 196.479])
+
         """
+        F_vol = self.F_vol
+        if F_vol == 0.: return 0.
+        if units is None:
+            return self.imol[IDs] / F_vol 
+        else:
+            num, denum = units.split('/')
+            return self.get_flow(num + '/hr', (phase, IDs)) / self.get_total_flow(denum + '/hr')
+        
         return self.imol[phase, IDs] / self.F_vol
     
     ### Equilibrium ###
     
+    # @property
+    # def equilibrium(self):
+    #     """[Equilibrium] An object that can perform equilibrium on the stream."""
+    #     phases = self.phases
+    #     if phases == ('g', 'l'):
+    #         cache = self._vle_cache
+    #     elif phases == ('l', 'L'):
+    #         cache = self._lle_cache
+    #     elif phases == ('l', 's'):
+    #         cache = self._sle_cache
+    #     else:
+    #         return None
+    #     return cache.retrieve()
+        
     @property
-    def vle(self):
-        """[VLE] An object that can perform vapor-liquid equilibrium on the stream."""
+    def vle(self) -> eq.VLE:
+        """An object that can perform vapor-liquid equilibrium on the stream."""
         phases = self.phases
         if 'l' not in phases or 'g' not in phases: 
             self.phases = [*phases, 'l', 'g']
         return self._vle_cache.retrieve()
     @property
-    def lle(self):
-        """[LLE] An object that can perform liquid-liquid equilibrium on the stream."""
+    def lle(self) -> eq.LLE:
+        """An object that can perform liquid-liquid equilibrium on the stream."""
         phases = self.phases
         if 'l' not in phases or 'L' not in phases: 
             self.phases = [*phases, 'l', 'L']
         return self._lle_cache.retrieve()
     @property
-    def sle(self):
-        """[SLE] An object that can perform solid-liquid equilibrium on the stream."""
+    def sle(self) -> eq.SLE:
+        """An object that can perform solid-liquid equilibrium on the stream."""
         phases = self.phases
         if 's' not in phases or 'l' not in phases: 
             self.phases = [*phases, 's', 'l']
@@ -922,15 +968,17 @@ class MultiStream(Stream):
             raise RuntimeError('multiple phases present; cannot convert to single phase stream')
     
     @property
-    def phase(self):
+    def phase(self) -> str:
         imol = self._imol
-        return ''.join([phases[0] for phases in ('gG', 'lL', 'sS') if not imol.phases_are_empty(phases)])
+        return ''.join([phases[0] for phases in ('g', 'lL', 'sS') if not imol.phases_are_empty(phases)])
     @phase.setter
     def phase(self, phase):
-        if len(phase) > 1: self.phases = phase
-        self._imol = self._imol.to_chemical_indexer(phase)
-        self._streams.clear()
-        self.__class__ = Stream
+        if len(phase) > 1: 
+            self.phases = phase
+        else:
+            self._imol = self._imol.to_chemical_indexer(phase)
+            self._streams.clear()
+            self.__class__ = Stream
     
     ### Representation ###
     
@@ -1029,4 +1077,4 @@ class MultiStream(Stream):
         price = utils.repr_kwarg('price', self.price)
         print(f"{type(self).__name__}(ID={repr(self.ID)}, {phases}, T={self.T:.2f}, "
               f"P={self.P:.6g}{price}{phase_data})")
-    
\ No newline at end of file
+    
