@@ -46,9 +46,7 @@ class Thermo:
     
     >>> import thermosteam as tmo
     >>> thermo = tmo.Thermo(['Ethanol', 'Water'], cache=True)
-    >>> thermo
-    Thermo(chemicals=CompiledChemicals([Ethanol, Water]), mixture=Mixture(rule='ideal', ..., include_excess_energies=False), Gamma=DortmundActivityCoefficients, Phi=IdealFugacityCoefficients, PCF=IdealPoyintingCorrectionFactors)
-    >>> thermo.show() # May be easier to read
+    >>> thermo.show() 
     Thermo(
         chemicals=CompiledChemicals([Ethanol, Water]),
         mixture=Mixture(
@@ -83,10 +81,10 @@ class Thermo:
     >>> stream.show()
     MultiStream: stream
      phases: ('g', 'l'), T: 361 K, P: 101325 Pa
-     flow (kmol/hr): (g) Ethanol  32.2
-                         Water    17.3
-                     (l) Ethanol  67.8
-                         Water    82.7
+     flow (kmol/hr): (g) Ethanol  32.6
+                         Water    17.6
+                     (l) Ethanol  67.4
+                         Water    82.4
     >>> # Modified Roult's law:                 
     >>> tmo.settings.set_thermo(thermo)
     >>> stream = tmo.Stream('stream', Water=100, Ethanol=100)
@@ -329,4 +327,4 @@ class IdealThermo:
               f"    chemicals={self.chemicals},\n"
               f"    mixture={mixture_info},\n"
                ")")
-    _ipython_display_ = show
\ No newline at end of file
+    _ipython_display_ = show
