@@ -249,6 +249,7 @@ class GroupActivityCoefficients(ActivityCoefficients):
         else:
             self = super().__new__(cls)
         index, chemgroups = get_chemgroups(chemicals, self.group_name)
+        if len(index) <= 1: return IdealActivityCoefficients(chemicals)
         self._index = index
         all_groups = set()
         for groups in chemgroups: all_groups.update(groups)
@@ -401,7 +402,7 @@ class DortmundActivityCoefficients(GroupActivityCoefficients):
     >>> # RuntimeWarning: O2 has no defined Dortmund groups; 
     >>> # functional group interactions are ignored
     >>> Gamma([0.5, 0.5], 350.) 
-    array([1., 1.])
+    1.0
     
     """
     __slots__ = ()
