@@ -42,7 +42,7 @@ def solve_x(x_guess, x_gamma_poyinting, T, f_gamma, gamma_args, f_pcf, pcf_args)
     args = (x_gamma_poyinting, T, f_gamma, gamma_args, f_pcf, pcf_args)
     try:
         x = flx.wegstein(x_iter, x_guess, 1e-10, args=args, checkiter=False,
-                          checkconvergence=False, convergenceiter=3)
+                         checkconvergence=False, convergenceiter=3)
     except:
         return x_gamma_poyinting
     return x
@@ -89,18 +89,20 @@ class DewPoint:
     >>> molar_composition = (0.5, 0.5)
     >>> dp = DP(z=molar_composition, T=355)
     >>> dp
-    DewPointValues(T=355.00, P=91970, IDs=('Water', 'Ethanol'), z=[0.5 0.5], x=[0.851 0.149])
+    DewPointValues(T=355.00, P=92086, IDs=('Water', 'Ethanol'), z=[0.5 0.5], x=[0.85 0.15])
     >>> # Note that the result is a DewPointValues object which contain all results as attibutes
     >>> (dp.T, round(dp.P), dp.IDs, dp.z, dp.x)
-    (355, 91970, ('Water', 'Ethanol'), array([0.5, 0.5]), array([0.851, 0.149]))
+    (355, 92086, ('Water', 'Ethanol'), array([0.5, 0.5]), array([0.85, 0.15]))
     >>> # Solve for dew point at constant pressure
     >>> DP(z=molar_composition, P=2*101324)
-    DewPointValues(T=376.26, P=202648, IDs=('Water', 'Ethanol'), z=[0.5 0.5], x=[0.832 0.168])
+    DewPointValues(T=376.23, P=202648, IDs=('Water', 'Ethanol'), z=[0.5 0.5], x=[0.832 0.168])
 
     """
     __slots__ = ('chemicals', 'phi', 'gamma', 'IDs', 
                  'pcf', 'Psats', 'Tmin', 'Tmax', 'Pmin', 'Pmax')
     _cached = {}
+    T_tol = 1e-9
+    P_tol = 1e-3
     def __init__(self, chemicals=(), thermo=None):
         thermo = settings.get_default_thermo(thermo)
         chemicals = tuple(chemicals)
@@ -151,8 +153,8 @@ class DewPoint:
     
     def _Tx_ideal(self, zP):
         f = self._T_error_ideal
-        Tmin = self.Tmin
-        Tmax = self.Tmax
+        Tmin = self.Tmin + 10.
+        Tmax = self.Tmax - 10.
         x = zP.copy() 
         args = (zP, x)
         fmin = f(Tmin, *args)
@@ -160,7 +162,7 @@ class DewPoint:
         fmax = f(Tmax, *args)
         if fmax < 0.: return Tmax, x
         T = flx.IQ_interpolation(f, Tmin, Tmax, fmin, fmax, 
-                                 None, 1e-9, 5e-12, args,
+                                 None, self.T_tol, 5e-12, args,
                                  checkiter=False, checkbounds=False)
         return T, x
     
@@ -205,8 +207,8 @@ class DewPoint:
         >>> chemicals = tmo.Chemicals(['Water', 'Ethanol'], cache=True)
         >>> tmo.settings.set_thermo(chemicals)
         >>> DP = tmo.equilibrium.DewPoint(chemicals)
-        >>> DP.solve_Tx(z=np.array([0.5, 0.5]), P=101325)
-        (357.451847, array([0.849, 0.151]))
+        >>> tmo.docround(DP.solve_Tx(z=np.array([0.5, 0.5]), P=101325))
+        (357.42, array([0.849, 0.151]))
         
         """
         positives = z > 0.
@@ -214,11 +216,9 @@ class DewPoint:
         if N == 0:
             raise ValueError('no positive components present')
         if N == 1:
-            T = self.chemicals[fn.first_true_index(positives)].Tsat(P)
+            T = self.chemicals[fn.first_true_index(positives)].Tsat(P, check_validity=False)
             x = z.copy()
         else:
-            if P > self.Pmax: P = self.Pmax
-            elif P < self.Pmin: P = self.Pmin
             f = self._T_error
             z_norm = z/z.sum()
             zP = z * P
@@ -226,7 +226,7 @@ class DewPoint:
             args = (P, z_norm, zP, x)
             try:
                 T = flx.aitken_secant(f, T_guess, T_guess + 1e-3,
-                                      1e-9, 5e-12, args,
+                                      self.T_tol, 5e-12, args,
                                       checkiter=False)
             except (InfeasibleRegion, DomainError):
                 Tmin = self.Tmin
@@ -262,8 +262,8 @@ class DewPoint:
         >>> chemicals = tmo.Chemicals(['Water', 'Ethanol'], cache=True)
         >>> tmo.settings.set_thermo(chemicals)
         >>> DP = tmo.equilibrium.DewPoint(chemicals)
-        >>> DP.solve_Px(z=np.array([0.5, 0.5]), T=352.28)
-        (82444.29876, array([0.853, 0.147]))
+        >>> tmo.docround(DP.solve_Px(z=np.array([0.5, 0.5]), T=352.28))
+        (82548.7827, array([0.852, 0.148]))
  
        """
         positives = z > 0.
@@ -274,8 +274,6 @@ class DewPoint:
             P = self.chemicals[fn.first_true_index(z)].Psat(T)
             x = z.copy()
         else:
-            if T > self.Tmax: T = self.Tmax
-            elif T < self.Tmin: T = self.Tmin
             z_norm = z/z.sum()
             Psats = np.array([i(T) for i in self.Psats], dtype=float)
             z_over_Psats = z/Psats
@@ -283,14 +281,14 @@ class DewPoint:
             args = (T, z_norm, z_over_Psats, x)
             f = self._P_error
             try:
-                P = flx.aitken_secant(f, P_guess, P_guess-10, 1e-3, 5e-12, args,
+                P = flx.aitken_secant(f, P_guess, P_guess-10, self.P_tol, 5e-12, args,
                                       checkiter=False)
             except (InfeasibleRegion, DomainError):
                 Pmin = self.Pmin
                 Pmax = self.Pmax
                 P = flx.IQ_interpolation(f, Pmin, Pmax, 
                                          f(Pmin, *args), f(Pmax, *args),
-                                         P_guess, 1e-3, 5e-12, args,
+                                         P_guess, self.P_tol, 5e-12, args,
                                          checkiter=False, checkbounds=False)
         return P, fn.normalize(x)
     
@@ -299,4 +297,4 @@ class DewPoint:
         return f"{type(self).__name__}([{chemicals}])"
     
 class DewPointCache(Cache): load = DewPoint
-del Cache
\ No newline at end of file
+del Cache
