@@ -12,7 +12,7 @@ __all__ = (
     'vle_domain',
 )
 
-Tmin_limit = 200.
+Tmin_limit = 50.
 Tmax_limit = 1000.
 
 def vle_domain(chemicals):
@@ -34,9 +34,10 @@ def vle_domain(chemicals):
 
     """
     Psats = [i.Psat for i in chemicals]
-    Tmax_Psat = min([i.Tmax for i in Psats])
+    Tmax_Psat = max([i.Tmax for i in Psats])
     Tmax = min(Tmax_Psat, Tmax_limit) - 1e-2
     
-    Tmin_Psat = max([i.Tmin for i in Psats])
+    Tmin_Psat = min([i.Tmin for i in Psats])
     Tmin = max(Tmin_Psat, Tmin_limit) + 1e-2
+    
     return (Tmin, Tmax)
\ No newline at end of file
