@@ -11,19 +11,18 @@ from numba import njit
 from ..utils import Cache
 from .equilibrium import Equilibrium
 from .binary_phase_fraction import phase_fraction
+from scipy.optimize import differential_evolution
 import numpy as np
 
 __all__ = ('LLE', 'LLECache')
 
-# @njit(cache=True)
+# TODO: SUBMIT ISSUE TO NUMBA
+@njit(cache=True)
 def liquid_activities(mol_L, T, f_gamma, gamma_args):
     total_mol_L = mol_L.sum()
-    if total_mol_L:
-        x = mol_L / total_mol_L
-        gamma = f_gamma(x, T, *gamma_args)
-        xgamma = x * gamma
-    else:
-        xgamma = np.ones_like(mol_L)
+    x = mol_L / total_mol_L
+    gamma = f_gamma(x, T, *gamma_args)
+    xgamma = x * gamma
     return xgamma
 
 @njit(cache=True)
@@ -32,17 +31,23 @@ def gibbs_free_energy_of_liquid(mol_L, xgamma):
     g_mix = (mol_L * np.log(xgamma)).sum()
     return g_mix
 
+@njit(cache=True)
 def lle_objective_function(mol_L, mol, T, f_gamma, gamma_args):
     mol_l = mol - mol_L
-    xgamma_l = liquid_activities(mol_l, T, f_gamma, gamma_args)
-    xgamma_L = liquid_activities(mol_L, T, f_gamma, gamma_args)
-    g_mix_l = gibbs_free_energy_of_liquid(mol_l, xgamma_l)
-    g_mix_L = gibbs_free_energy_of_liquid(mol_L, xgamma_L)
+    if mol_l.sum() == 0.:
+        g_mix_l = 0.
+    else:
+        xgamma_l = liquid_activities(mol_l, T, f_gamma, gamma_args)
+        g_mix_l = gibbs_free_energy_of_liquid(mol_l, xgamma_l)
+    if mol_L.sum() == 0.:
+        g_mix_L = 0.
+    else:
+        xgamma_L = liquid_activities(mol_L, T, f_gamma, gamma_args)
+        g_mix_L = gibbs_free_energy_of_liquid(mol_L, xgamma_L)
     g_mix = g_mix_l + g_mix_L
     return g_mix
 
 def solve_lle_liquid_mol(mol, T, f_gamma, gamma_args, **differential_evolution_options):
-    from scipy.optimize import differential_evolution
     args = (mol, T, f_gamma, gamma_args)
     bounds = np.zeros([mol.size, 2])
     bounds[:, 1] = mol
@@ -72,7 +77,8 @@ class LLE(Equilibrium, phases='lL'):
     >>> settings.set_thermo(['Water', 'Ethanol', 'Octane', 'Hexane'], cache=True)
     >>> imol = indexer.MolarFlowIndexer(
     ...             l=[('Water', 304), ('Ethanol', 30)],
-    ...             L=[('Octane', 40), ('Hexane', 1)])
+    ...             L=[('Octane', 40), ('Hexane', 1)]
+    ... )
     >>> lle = equilibrium.LLE(imol)
     >>> lle(T=360)
     >>> lle
@@ -93,17 +99,17 @@ class LLE(Equilibrium, phases='lL'):
     )
     differential_evolution_options = {'seed': 0,
                                       'popsize': 12,
-                                      'tol': 0.002}
+                                      'tol': 1e-6}
     
     def __init__(self, imol=None, thermal_condition=None, thermo=None,
-                 composition_cache_tolerance=1e-6,
-                 temperature_cache_tolerance=1e-6):
+                 composition_cache_tolerance=1e-5,
+                 temperature_cache_tolerance=1e-3):
         super().__init__(imol, thermal_condition, thermo)
         self.composition_cache_tolerance = composition_cache_tolerance
         self.temperature_cache_tolerance = temperature_cache_tolerance
         self._lle_chemicals = None
     
-    def __call__(self, T, P=None, top_chemical=None):
+    def __call__(self, T, P=None, top_chemical=None, update=True):
         """
         Perform liquid-liquid equilibrium.
 
@@ -114,24 +120,29 @@ class LLE(Equilibrium, phases='lL'):
         P : float, optional
             Operating pressure [Pa].
         top_chemical : str, optional
-            Identifier of chemical that will be favored in the "liquid" phase.
+            Identifier of chemical that will be favored in the "LIQUID" phase.
+        update : bool, optional
+            Whether to update material flows, temperature and pressure. If False,
+            returns the chemicals in liquid-liquid equilibrium, 
+            partition coefficients, and phase fraction.
             
         """
-        thermal_condition = self._thermal_condition
-        thermal_condition.T = T
-        if P: thermal_condition.P = P
+        if update:
+            thermal_condition = self._thermal_condition
+            thermal_condition.T = T
+            if P: thermal_condition.P = P
         imol = self._imol
         mol, index, lle_chemicals = self.get_liquid_mol_data()
         F_mol = mol.sum()
-        if F_mol:
-            z_mol = mol / F_mol
+        if F_mol and len(lle_chemicals) > 1:
+            z_mol = mol = mol / F_mol # Normalize first
             if (self._lle_chemicals == lle_chemicals 
                 and T - self._T < self.temperature_cache_tolerance 
                 and (self._z_mol - z_mol < self.composition_cache_tolerance).all()):
                 K = self._K 
-                phi = phase_fraction(z_mol, K, self._phi)
+                self._phi = phi = phase_fraction(z_mol, K, self._phi)
                 y = z_mol * K / (phi * K + (1 - phi))
-                mol_l = y * phi * F_mol
+                mol_l = y * phi
                 mol_L = mol - mol_l
             else:
                 gamma = self.thermo.Gamma(lle_chemicals)
@@ -146,21 +157,21 @@ class LLE(Equilibrium, phases='lL'):
                     top_chemical_index = IDs[top_chemical]
                     C_L = mass_L[top_chemical_index] / mass_L.sum()
                     C_l = mass_l[top_chemical_index] / mass_l.sum()
-                    top_L = C_L > C_l
-                    if top_L: mol_l, mol_L = mol_L, mol_l
+                    if C_L < C_l: mol_l, mol_L = mol_L, mol_l
                 F_mol_l = mol_l.sum()
                 z_mol_l = mol_l / F_mol_l
                 F_mol_L = mol_L.sum()
                 z_mol_L = mol_L / F_mol_L
-                z_mol_L[z_mol_L < 1e-16] = 1e-16
-                K = z_mol_l / z_mol_L
+                z_mol_l[z_mol_l < 1e-16] = 1e-16
+                K = z_mol_L / z_mol_l
                 self._K = K
-                self._phi = F_mol_l / (F_mol_l + F_mol_L)
+                self._phi = F_mol_L / (F_mol_L + F_mol_l)
                 self._lle_chemicals = lle_chemicals
                 self._z_mol = z_mol
                 self._T = T
-            imol['l'][index] = mol_l
-            imol['L'][index] = mol_L
+            if not update: return self._lle_chemicals, self._K, self._phi
+            imol['l'][index] = mol_l * F_mol
+            imol['L'][index] = mol_L * F_mol
         
     def get_liquid_mol_data(self):
         # Get flow rates
