@@ -10,9 +10,9 @@
 import thermosteam as tmo
 from .units_of_measure import AbsoluteUnitsOfMeasure
 from . import utils
-from .exceptions import UndefinedChemical
+from .exceptions import UndefinedChemicalAlias
 from .base import PhaseHandle
-from ._phase import Phase, LockedPhase, NoPhase, PhaseIndexer, phase_tuple, check_phase
+from ._phase import Phase, LockedPhase, NoPhase, PhaseIndexer, phase_tuple
 from free_properties import PropertyFactory, property_array
 from collections.abc import Iterable
 import numpy as np
@@ -31,6 +31,14 @@ __all__ = (
     'VolumetricFlowProperty'
 )
 
+phase_names = {
+    'g': 'Gas',
+    'l': 'Liquid',
+    's': 'Solid',
+    'L': 'LIQUID',
+    'S': 'SOLID',
+}
+
 # %% Utilities
 
 _new = object.__new__
@@ -38,7 +46,7 @@ _new = object.__new__
 def raise_material_indexer_index_error():
     raise IndexError("index by [phase, IDs] where phase is a "
                      "(str, ellipsis, or missing), and IDs is a "
-                     "(str, tuple(str), ellipisis, or missing)")
+                     "(str, Sequence[str], ellipisis, or missing)")
 
 def find_main_phase(indexers, default):
     main_indexer, *indexers = indexers
@@ -145,7 +153,7 @@ class SplitIndexer(Indexer):
             data[:] = 0.
         for CAS, split in zip(self._chemicals.CASs, old_data):
             if CAS in chemicals: data[chemicals.index(CAS)] = split
-        self._load_chemicals(chemicals)
+        self._chemicals = chemicals
         return old_data
     
     @classmethod
@@ -253,7 +261,7 @@ class ChemicalIndexer(Indexer):
     
     Parameters
     ----------
-    phase : [str or PhaseContainer] {'s', 'l', 'g', 'S', 'L', 'G'}
+    phase : [str or PhaseContainer] {'s', 'l', 'g', 'S', 'L'}
         Phase of data.
     units : str
         Units of measure of input data. 
@@ -292,7 +300,7 @@ class ChemicalIndexer(Indexer):
             data[:] = 0.
         for CAS, value in zip(self._chemicals.CASs, old_data):
             if value: data[chemicals.index(CAS)] = value
-        self._load_chemicals(chemicals)
+        self._chemicals = chemicals
         return old_container
     
     def __reduce__(self):
@@ -424,7 +432,7 @@ class ChemicalIndexer(Indexer):
         """Return phase and composition."""
         data = self._data
         total = data.sum()
-        if total <= 0.: raise RuntimeError(f"'{tmo.settings.phase_names[self.phase]}' phase does not exist")
+        if total <= 0.: raise RuntimeError(f"'{phase_names[self.phase]}' phase does not exist")
         return self.phase, data / total
     
     def __format__(self, tabs=""):
@@ -790,12 +798,12 @@ class MaterialIndexer(Indexer):
     
     def _get_index_data(self, key):
         cache = self._index_cache
-        try: 
+        try:
             index_data = cache[key]
         except KeyError:
             try:
                 index, kind = self._chemicals._get_index_and_kind(key)
-            except UndefinedChemical as error:
+            except UndefinedChemicalAlias as error:
                 index, kind = self._get_index_and_kind(key, error)
                 sum_across_phases = False
             else:
@@ -803,7 +811,21 @@ class MaterialIndexer(Indexer):
             cache[key] = index_data = (index, kind, sum_across_phases)
             utils.trim_cache(cache)
         except TypeError:
-            raise TypeError("only strings, tuples, and ellipsis are valid index keys")
+            try:
+                key = tuple([i if i.__hash__ else tuple(i) for i in key])
+                index_data = cache[key]
+            except KeyError:
+                try:
+                    index, kind = self._chemicals._get_index_and_kind(key)
+                except UndefinedChemicalAlias as error:
+                    index, kind = self._get_index_and_kind(key, error)
+                    sum_across_phases = False
+                else:
+                    sum_across_phases = True
+                cache[key] = index_data = (index, kind, sum_across_phases)
+                utils.trim_cache(cache)
+            except TypeError:
+                raise TypeError("only strings, sequences of strings, and ellipsis are valid index keys")
         return index_data
     
     def _get_index_and_kind(self, phase_IDs, undefined_chemical_error):
@@ -1077,7 +1099,7 @@ def by_volume(self, TP):
         for i, phase in enumerate(phases):
             for j, chem in enumerate(chemicals):
                 index = i, j
-                phase_name = tmo.settings._phase_names[phase]
+                phase_name = phase_names[phase]
                 vol[index] = VolumetricFlowProperty(f"{phase_name}{chem.ID}", 
                                                     mol, index, chem.V, TP, phase)
         self._data_cache[TP] = \
