@@ -9,10 +9,16 @@
 """
 import flexsolve as flx
 import numpy as np
+from math import exp
 from thermosteam import functional as fn
 from .. import units_of_measure as thermo_units
 from ..base import PhaseMixtureHandle
-from .ideal_mixture_model import IdealTMixtureModel, IdealTPMixtureModel
+from .ideal_mixture_model import (
+    IdealTMixtureModel, 
+    IdealTPMixtureModel, 
+    IdealEntropyModel, 
+    IdealHvapModel
+)
 from .._chemicals import Chemical, CompiledChemicals, chemical_data_array
 
 __all__ = ('Mixture',)
@@ -36,36 +42,46 @@ def group_handles_by_phase(phase_handles):
             handles.append(prop)
     return handles_by_phase
     
-def build_ideal_PhaseTMixtureHandle(chemicals, var):
+def build_ideal_PhaseMixtureHandle(chemicals, var, Model):
     setfield = object.__setattr__
     getfield = getattr
     phase_handles = [getfield(i, var) for i in chemicals]
     new = PhaseMixtureHandle.__new__(PhaseMixtureHandle)
     for phase, handles in group_handles_by_phase(phase_handles).items():
-        setfield(new, phase, IdealTMixtureModel(handles, var))
-    setfield(new, 'var', var)
-    return new
-
-def build_ideal_PhaseTPMixtureHandle(chemicals, var):
-    setfield = object.__setattr__
-    getfield = getattr
-    phase_handles = [getfield(i, var) for i in chemicals]
-    new = PhaseMixtureHandle.__new__(PhaseMixtureHandle)
-    for phase, handles in group_handles_by_phase(phase_handles).items():
-        setfield(new, phase, IdealTPMixtureModel(handles, var))
+        setfield(new, phase, Model(handles, var))
     setfield(new, 'var', var)
     return new
 
 # %% Energy balance
 
-def iter_temperature(T, H, H_model, phase, mol, P, Cn):
-    # Used to solve for ethalpy at given temperature
+def iter_T_at_HP(T, H, H_model, phase, mol, P, Cn_model, Cn_cache):
+    # Used to solve for temperature at given ethalpy 
+    counter, Cn = Cn_cache
+    if not counter % 5: Cn_cache[1] = Cn = Cn_model(phase, mol, T)
+    Cn_cache[0] += 1
     return T + (H - H_model(phase, mol, T, P)) / Cn
 
-def xiter_temperature(T, H, H_model, phase_mol, P, Cn):
-    # Used to solve for ethalpy at given temperature
+def xiter_T_at_HP(T, H, H_model, phase_mol, P, Cn_model, Cn_cache):
+    # Used to solve for temperature at given ethalpy 
+    counter, Cn = Cn_cache
+    if not counter % 5: Cn_cache[1] = Cn = Cn_model(phase_mol, T)
+    Cn_cache[0] += 1
     return T + (H - H_model(phase_mol, T, P)) / Cn
 
+def iter_T_at_SP(T, S, S_model, phase, mol, P, Cn_model, Cn_cache):
+    # Used to solve for temperature at given entropy 
+    counter, Cn = Cn_cache
+    if not counter % 5: Cn_cache[1] = Cn = Cn_model(phase, mol, T)
+    Cn_cache[0] += 1
+    return T * exp((S - S_model(phase, mol, T, P)) / Cn)
+
+def xiter_T_at_SP(T, S, S_model, phase_mol, P, Cn_model, Cn_cache):
+    # Used to solve for temperature at given entropy 
+    counter, Cn = Cn_cache
+    if not counter % 5: Cn_cache[1] = Cn = Cn_model(phase_mol, T)
+    Cn_cache[0] += 1
+    return T * exp((S - S_model(phase_mol, T, P)) / Cn)
+
 
 # %% Ideal mixture
 
@@ -78,7 +94,7 @@ class Mixture:
     rule : str
         Description of mixing rules used.
     Cn : function(phase, mol, T)
-        Molar heat capacity mixture model [J/mol/K].
+        Molar isobaric heat capacity mixture model [J/mol/K].
     H : function(phase, mol, T)
         Enthalpy mixture model [J/mol].
     S : function(phase, mol, T, P)
@@ -124,7 +140,7 @@ class Mixture:
         Whether to include excess energies
         in enthalpy and entropy calculations.
     Cn(phase, mol, T) : 
-        Mixture molar heat capacity [J/mol/K].
+        Mixture molar isobaric heat capacity [J/mol/K].
     mu(phase, mol, T, P) : 
         Mixture dynamic viscosity [Pa*s].
     V(phase, mol, T, P) : 
@@ -208,7 +224,6 @@ class Mixture:
         752.513
         
         """
-        
         if rule == 'ideal':
             isa = isinstance
             if isa(chemicals, CompiledChemicals):
@@ -218,15 +233,15 @@ class Mixture:
                 chemicals = [(i if isa(i, Chemical) else Chemical(i)) for i in chemicals]
                 MWs = chemical_data_array(chemicals, 'MW')
             getfield = getattr
-            Cn =  build_ideal_PhaseTMixtureHandle(chemicals, 'Cn')
-            H =  build_ideal_PhaseTPMixtureHandle(chemicals, 'H')
-            S = build_ideal_PhaseTPMixtureHandle(chemicals, 'S')
-            H_excess = build_ideal_PhaseTPMixtureHandle(chemicals, 'H_excess')
-            S_excess = build_ideal_PhaseTPMixtureHandle(chemicals, 'S_excess')
-            mu = build_ideal_PhaseTPMixtureHandle(chemicals, 'mu')
-            V = build_ideal_PhaseTPMixtureHandle(chemicals, 'V')
-            kappa = build_ideal_PhaseTPMixtureHandle(chemicals, 'kappa')
-            Hvap = IdealTMixtureModel([getfield(i, 'Hvap') for i in chemicals], 'Hvap')
+            Cn =  build_ideal_PhaseMixtureHandle(chemicals, 'Cn', IdealTMixtureModel)
+            H =  build_ideal_PhaseMixtureHandle(chemicals, 'H', IdealTMixtureModel)
+            S = build_ideal_PhaseMixtureHandle(chemicals, 'S', IdealEntropyModel)
+            H_excess = build_ideal_PhaseMixtureHandle(chemicals, 'H_excess', IdealTPMixtureModel)
+            S_excess = build_ideal_PhaseMixtureHandle(chemicals, 'S_excess', IdealTPMixtureModel)
+            mu = build_ideal_PhaseMixtureHandle(chemicals, 'mu', IdealTPMixtureModel)
+            V = build_ideal_PhaseMixtureHandle(chemicals, 'V', IdealTPMixtureModel)
+            kappa = build_ideal_PhaseMixtureHandle(chemicals, 'kappa', IdealTPMixtureModel)
+            Hvap = IdealHvapModel(chemicals)
             sigma = IdealTMixtureModel([getfield(i, 'sigma') for i in chemicals], 'sigma')
             epsilon = IdealTMixtureModel([getfield(i, 'epsilon') for i in chemicals], 'epsilon')
             return cls(rule, Cn, H, S, H_excess, S_excess,
@@ -246,7 +261,7 @@ class Mixture:
         return fn.V_to_rho(self.V(phase, mol, T, P), MW) if MW else 0.
     
     def Cp(self, phase, mol, T):
-        """Mixture heat capacity [J/g/K]"""
+        """Mixture isobaric heat capacity [J/g/K]"""
         MW = self.MW(mol)
         return self.Cn(phase, mol, T) / MW if MW else 0.
     
@@ -275,7 +290,7 @@ class Mixture:
         return sum([self.rho(phase, mol, T, P) for phase, mol in phase_mol])
     
     def xCp(self, phase_mol, T):
-        """Multi-phase mixture heat capacity [kg/m3]."""
+        """Multi-phase mixture isobaric heat capacity [J/g/K]."""
         return sum([self.Cp(phase, mol, T) for phase, mol in phase_mol])
     
     def xalpha(self, phase_mol, T, P):
@@ -315,30 +330,42 @@ class Mixture:
     def H(self, phase, mol, T, P):
         """Return enthalpy [J/mol]."""
         H = self._H(phase, mol, T)
-        if self.include_excess_energies:
-            H += self._H_excess(phase, mol, T, P)
+        if self.include_excess_energies: H += self._H_excess(phase, mol, T, P)
         return H
     
     def S(self, phase, mol, T, P):
-        """Return entropy in [J/mol]."""
+        """Return entropy in [J/mol/K]."""
+        total_mol = mol.sum()
+        if total_mol == 0. : return 0.
         S = self._S(phase, mol, T, P)
         if self.include_excess_energies:
             S += self._S_excess(phase, mol, T, P)
         return S
     
-    def solve_T(self, phase, mol, H, T_guess, P):
+    def solve_T_at_HP(self, phase, mol, H, T_guess, P):
         """Solve for temperature in Kelvin."""
-        args = (H, self.H, phase, mol, P, self.Cn(phase, mol, T_guess))
-        return flx.aitken(iter_temperature, T_guess, 1e-6, args, 50, checkiter=False)
+        args = (H, self.H, phase, mol, P, self.Cn, [0, None])
+        return flx.aitken(iter_T_at_HP, T_guess, 1e-6, args, 50, checkiter=True)
         
-    def xsolve_T(self, phase_mol, H, T_guess, P):
+    def xsolve_T_at_HP(self, phase_mol, H, T_guess, P):
         """Solve for temperature in Kelvin."""
         phase_mol = tuple(phase_mol)
-        args = (H, self.xH, phase_mol, P, self.xCn(phase_mol, T_guess))
-        return flx.aitken(xiter_temperature, T_guess, 1e-6, args, 50, checkiter=False)
+        args = (H, self.xH, phase_mol, P, self.xCn, [0, None])
+        return flx.aitken(xiter_T_at_HP, T_guess, 1e-6, args, 50, checkiter=True)
+    
+    def solve_T_at_SP(self, phase, mol, S, T_guess, P):
+        """Solve for temperature in Kelvin."""
+        args = (S, self.S, phase, mol, P, self.Cn, [0, None])
+        return flx.aitken(iter_T_at_SP, T_guess, 1e-6, args, 50, checkiter=True)
         
-    def xCn(self, phase_mol, T):
-        """Multi-phase mixture heat capacity [J/mol/K]."""
+    def xsolve_T_at_SP(self, phase_mol, S, T_guess, P):
+        """Solve for temperature in Kelvin."""
+        phase_mol = tuple(phase_mol)
+        args = (S, self.xS, phase_mol, P, self.xCn, [0, None])
+        return flx.aitken(xiter_T_at_SP, T_guess, 1e-6, args, 50, checkiter=True)
+    
+    def xCn(self, phase_mol, T, P=None):
+        """Multi-phase mixture molar isobaric heat capacity [J/mol/K]."""
         return sum([self.Cn(phase, mol, T) for phase, mol in phase_mol])
     
     def xH(self, phase_mol, T, P):
@@ -351,7 +378,7 @@ class Mixture:
         return H_total
     
     def xS(self, phase_mol, T, P):
-        """Multi-phase mixture entropy [J/mol]."""
+        """Multi-phase mixture entropy [J/mol/K]."""
         S = self._S
         S_total = sum([S(phase, mol, T, P) for phase, mol in phase_mol])
         if self.include_excess_energies:
