@@ -389,8 +389,18 @@ class Reaction:
         if tmo.reaction.CHECK_FEASIBILITY:
             negatives = values < 0.
             if negatives.any():
-                if values[negatives].sum() < -1e-16:
-                    raise InfeasibleRegion('not enough reactants; reaction conversion')
+                negative_values = values[negatives]
+                if negative_values.sum() < -1e-12:
+                    X_net = self.X_net()
+                    for ID, X in X_net.items():
+                        if X > 1.: RuntimeError(f"conversion of '{ID}' is over 100%")
+                    if negatives.ndim == 2:
+                        negatives = negatives.any(axis=1)
+                    pos, = np.where(negatives)
+                    chemicals = self.chemicals.tuple
+                    IDs = [chemicals[i].ID for i in pos]
+                    if len(IDs) == 1: IDs = repr(IDs[0])
+                    raise InfeasibleRegion(f'conversion of {IDs} is over 100%; reaction conversion')
                 else:
                     values[negatives] = 0.
         else:
@@ -509,7 +519,7 @@ class Reaction:
                          CH4  4.5
                          O2   96
                          CO2  0.5
-                         H2O  108    
+                         H2O  108
         
         Sequential combustion of CH4 and CO:
         
@@ -555,6 +565,17 @@ class Reaction:
     def _reaction(self, material_array):
         material_array += material_array[self._X_index] * self.X * self._stoichiometry
     
+    def X_net(self, indexer=False):
+        """Return net reaction conversion of reactants as a dictionary or
+        a ChemicalIndexer if indexer is True."""
+        X_net = {self.reactant: self.X}
+        if indexer:
+            chemicals = self.chemicals
+            data = chemicals.kwarray(X_net)
+            return ChemicalIndexer.from_data(data, NoPhase, chemicals, False)
+        else:
+            return X_net
+    
     @property
     def dH(self):
         """
@@ -563,17 +584,48 @@ class Reaction:
         
         Warning
         -------
-        Latents heats of vaporization are not accounted for; only heats of 
-        formation are included in this term. Note that heats of vaporization
-        are temperature dependent and cannot be calculated using a Reaction
-        object.
-        
+        This property also accounts for:
+        * Latent heats (at 298.15 K) when phases are specified.
+        * The extent of reaction, `X`.
+        * The basis of the reaction (by mol or by wt)
+
         """
-        if self._basis == 'mol':
-            Hfs = self._chemicals.Hf
-        else:
-            Hfs = self._chemicals.Hf / self.MWs
-        return self.X * (Hfs * self._stoichiometry).sum()
+        chemicals = self._chemicals
+        stoichiometry = self._stoichiometry
+        phases = self.phases
+        Hfs = chemicals.Hf
+        if phases:
+            H_latent = np.zeros_like(stoichiometry)
+            for i, phase in enumerate(self.phases):
+                for j, chemical in enumerate(chemicals):
+                    phase_ref = chemical.phase_ref
+                    if phase_ref != phase and stoichiometry[i, j] != 0.:
+                        if phase_ref == 'l':
+                            if phase == 'g':
+                                H_latent[i, j] = chemical.Hvap(298.15)
+                            elif phase == 's':
+                                H_latent[i, j] = -chemical.Hfus
+                            else:
+                                raise RuntimeError(f"invalid phase '{phase}'")
+                        elif phase_ref == 'g':
+                            if phase == 'l':
+                                H_latent[i, j] = -chemical.Hvap(298.15)
+                            elif phase == 's':
+                                H_latent[i, j] = -(chemical.Hvap(298.15) + chemical.Hfus)
+                            else:
+                                raise RuntimeError(f"invalid phase '{phase}'")
+                        elif phase_ref == 's':
+                            if phase == 'l':
+                                H_latent[i, j] = chemical.Hfus
+                            elif phase == 'g':
+                                H_latent[i, j] = chemical.Hfus + chemical.Hvap(298.15)
+                            else:
+                                raise RuntimeError(f"invalid phase '{phase}'")
+                        else:
+                            raise RuntimeError(f"invalid reference phase '{phase_ref}'")
+            Hfs = Hfs + H_latent
+        if self._basis == 'wt': Hfs = Hfs / self.MWs
+        return self.X * (Hfs * stoichiometry).sum()
     
     @property
     def X(self):
@@ -819,6 +871,9 @@ class Reaction:
         columns = [f'Stoichiometry (by {self.basis})', 'Reactant', 'Conversion [%]']
         stoichiometry = get_stoichiometric_string(self.stoichiometry, self.phases, self.chemicals)
         reactant = self.reactant
+        if self.phases: 
+            phase, reactant = reactant
+            reactant += ',' + phase
         conversion = 100. * self.X
         df = pd.DataFrame(data=[[stoichiometry, reactant, conversion]], columns=columns, index=[index] if index else None)
         df.index.name = 'Reaction'
@@ -964,6 +1019,14 @@ class ReactionSet:
             rxnset._parent_index = (self, index)
             return rxnset
     
+    def _rescale(self):
+        """Scale stoichiometry to a per reactant basis."""
+        stoichiometry = self._stoichiometry
+        M, N = stoichiometry.shape
+        new_scale = np.zeros([M, 1])
+        for index in enumerate(self._X_index): new_scale[index[0], 0] = -stoichiometry[index] 
+        self._stoichiometry /= new_scale
+    
     def reset_chemicals(self, chemicals):
         if chemicals is self._chemicals: return
         if hasattr(self, '_parent_index'):
@@ -985,7 +1048,7 @@ class ReactionSet:
                     for k in range(C):
                         value = stoichiometry[i, j, k]
                         if value: new_stoichiometry[i, j, chemicals.index(IDs[k])] = value
-            X_index = tuple([(phases.index(i), chemicals.index(j)) for i, j in reactants])
+            X_index = [(phases.index(i), chemicals.index(j)) for i, j in reactants]
         else:
             A, B = stoichiometry.shape
             new_stoichiometry = np.zeros([A, chemicals.size])
@@ -994,7 +1057,7 @@ class ReactionSet:
                 for j in range(B):
                     value = stoichiometry[i, j]
                     if value: new_stoichiometry[i, chemicals.index(IDs[j])] = value
-            X_index = tuple([chemicals.index(i) for i in reactants])
+            X_index = [chemicals.index(i) for i in reactants]
         self._chemicals = chemicals
         self._stoichiometry = new_stoichiometry
         self._X_index = X_index
@@ -1380,9 +1443,13 @@ class ReactionSystem:
         
     force_reaction = Reaction.force_reaction
     adiabatic_reaction = Reaction.adiabatic_reaction
+    chemicals = Reaction.chemicals
     __call__ = Reaction.__call__
     show = Reaction.show
     
+    def __getitem__(self, index):
+        return self._reactions[index]
+    
     @property
     def X(self):
         return [i.X for i in self._reactions]
@@ -1489,4 +1556,4 @@ RxnI = ReactionItem
 RxnS = ReactionSet
 PRxn = ParallelReaction
 SRxn = SeriesReaction
-RxnSys = ReactionSystem
\ No newline at end of file
+RxnSys = ReactionSystem
