@@ -31,7 +31,9 @@ def check_valid_ID(ID):
 
 class Registry: # pragma: no cover
 
-    __slots__ = ('data', 'safe_to_replace', '_dumps')
+    __slots__ = ('data', 'safe_to_replace', 'context_levels')
+
+    AUTORENAME = False #: Whether to rename objects with conflicting IDs
 
     def untrack(self, objs):
         """
@@ -44,9 +46,13 @@ class Registry: # pragma: no cover
     def track(self, objs):
         """
         Reregister objects if they are not already in the registry and mark 
-        objects as unsafe to replace, so that warnings can be issued if any are 
+        objects as unsafe to replace so that warnings can be issued if any are 
         replaced.
         
+        Warning
+        -------
+        Overrides old objects with the same ID.
+        
         """
         safe_to_replace = self.safe_to_replace
         data = self.data
@@ -57,7 +63,7 @@ class Registry: # pragma: no cover
     def __init__(self, objs=None):
         self.data = {i.ID: i for i in objs} if objs else {}
         self.safe_to_replace = set()
-        self._dumps = {}
+        self.context_levels = []
 
     def search(self, ID):
         """Return object given ID. If ID not in registry, return None."""
@@ -82,7 +88,16 @@ class Registry: # pragma: no cover
         if ID in data:
             other = data[ID]
             if obj is not other and other not in self.safe_to_replace:
-                if ID_old:
+                if self.AUTORENAME:
+                    try:
+                        base, num = ID.rsplit('_', 1)
+                        num = int(num)
+                        other.ID = ID + '_' + str(num + 1) 
+                        ID += '_' + str(num + 2)
+                    except:
+                        other.ID = ID + '_1'
+                        ID += '_2'
+                elif ID_old:
                     warning = RuntimeWarning(
                         f"upon renaming, {repr(obj)} replaced {repr(other)} "
                          "in registry"
@@ -108,15 +123,27 @@ class Registry: # pragma: no cover
         
     def _close_registration(self, ID, obj):
         self.data[ID] = obj
-        for i in self._dumps.values(): 
-            if obj not in i: i.append(obj)
+        for i in self.context_levels: i.append(obj)
         obj._ID = ID
     
-    def _open_dump(self, key):
-        self._dumps[key] = []
+    def open_context_level(self):
+        self.context_levels.append([])
         
-    def _close_dump(self, key):
-        return self._dumps.pop(key)
+    def close_context_level(self):
+        return self.context_levels.pop()
+    
+    def get_context_level(self, level=None):
+        if level is None:
+            objs = list(self)
+        else:
+            context_levels = self.context_levels
+            N_levels = len(context_levels)
+            if (level > 0 and level >= N_levels
+                or level < 0 and -level > N_levels):
+                objs = list(self)
+            else:
+                objs = context_levels[level]
+        return objs
     
     def clear(self):
         """Clear data."""
@@ -133,7 +160,7 @@ class Registry: # pragma: no cover
                 ID = obj
                 obj = data[obj]
                 del data[ID]
-        for dump in self._dumps.values():
+        for dump in self.context_levels:
             if obj in dump: dump.remove(obj)
     
     def pop(self, obj):
@@ -147,7 +174,7 @@ class Registry: # pragma: no cover
                 ID = obj
                 obj = data[obj]
                 del data[ID]
-        for dump in self._dumps.values():
+        for dump in self.context_levels:
             if obj in dump: dump.remove(obj)
         return obj
     
